@@ -33,10 +33,9 @@
 #define CACHEDIR "cache"
 #define TMPDIR "tmp"
 #define PIDFILE PERMDATADIR"/unrealircd.pid"
+#define CONTROLFILE PERMDATADIR"/unrealircd.ctl"
 #define NO_U_TYPES
 #define NEED_U_INT32_T
-#define PREFIX_AQ
-#define LIST_SHOW_MODES
 #define strcasecmp _stricmp
 #define strncasecmp _strnicmp
 #define HAVE_EXPLICIT_BZERO
@@ -57,13 +56,13 @@
 #define _WIN32_WINNT 0x0601
 
 /* Generation version number (e.g.: 3 for Unreal3*) */
-#define UNREAL_VERSION_GENERATION 5
+#define UNREAL_VERSION_GENERATION 6
 
 /* Major version number (e.g.: 2 for Unreal3.2*) */
-#define UNREAL_VERSION_MAJOR 2
+#define UNREAL_VERSION_MAJOR 0
 
 /* Minor version number (e.g.: 1 for Unreal3.2.1) */
-#define UNREAL_VERSION_MINOR 4
+#define UNREAL_VERSION_MINOR 3
 
 /* Version suffix such as a beta marker or release candidate marker. (e.g.:
    -rcX for unrealircd-3.2.9-rcX) */
