@@ -109,7 +109,7 @@ char *find_best_asan_log(void)
 		}
 	}
 	closedir(fd);
-	return BadPtr(best_fname) ? NULL : best_fname;
+	return *best_fname ? best_fname : NULL;
 #else
 	return NULL;
 #endif
@@ -531,7 +531,7 @@ char *generate_crash_report(char *coredump, int *thirdpartymods)
 
 #define CRASH_REPORT_HOST "crash.unrealircd.org"
 
-SSL_CTX *crashreport_init_ssl(void)
+SSL_CTX *crashreport_init_tls(void)
 {
 	SSL_CTX *ctx_client;
 	char buf[512];
@@ -587,7 +587,7 @@ int crashreport_send(char *fname)
 	                           delimiter);
 	snprintf(footer, sizeof(footer), "\r\n--%s--\r\n", delimiter);
 
-	ctx_client = crashreport_init_ssl();
+	ctx_client = crashreport_init_tls();
 	if (!ctx_client)
 	{
 		printf("ERROR: TLS initalization failure (I)\n");
@@ -765,22 +765,6 @@ void report_crash(void)
 	if (!fname)
 		return;
 
-	if (time(NULL) > 1688169600)
-	{
-		printf("Crash report generated in '%s' but NOT sent.\n\n"
-		       "UnrealIRCd 5 is no longer supported since July 1, 2023.\n"
-		       "All support stopped after that date. You had 18+ months to upgrade.\n"
-		       "See https://www.unrealircd.org/docs/UnrealIRCd_5_EOL.\n",
-		       fname);
-		return;
-	} else
-	if (time(NULL) > 1656633600)
-	{
-		printf("WARNING: UnrealIRCd 5 is only receiving security fixes until July 1, 2023.\n"
-		       "         See https://www.unrealircd.org/docs/UnrealIRCd_5_EOL\n");
-		/* continue below.. */
-	}
-
 	if (thirdpartymods == 0)
 		thirdpartymods = check_third_party_mods_present();
 #ifndef _WIN32
