@@ -26,7 +26,7 @@ ModuleHeader MOD_HEADER
 	"5.0",
 	"Check connecting users against DNS Blacklists",
 	"UnrealIRCd Team",
-	"unrealircd-5",
+	"unrealircd-6",
 };
 
 /* In this module and the config syntax I tried to 'abstract' things
@@ -83,6 +83,9 @@ struct BLUser {
 	long save_tkltime;
 	char *save_opernotice;
 	char *save_reason;
+	char *save_blacklist;
+	char *save_blacklist_dns_name;
+	int save_blacklist_dns_reply;
 };
 
 /* Global variables */
@@ -96,7 +99,8 @@ void blacklist_free_conf(void);
 void delete_blacklist_block(Blacklist *e);
 void blacklist_md_free(ModData *md);
 int blacklist_handshake(Client *client);
-int blacklist_quit(Client *client, MessageTag *mtags, char *comment);
+int blacklist_ip_change(Client *client, const char *oldip);
+int blacklist_quit(Client *client, MessageTag *mtags, const char *comment);
 int blacklist_preconnect(Client *client);
 void blacklist_resolver_callback(void *arg, int status, int timeouts, struct hostent *he);
 int blacklist_start_check(Client *client);
@@ -109,13 +113,11 @@ void blacklist_free_bluser_if_able(BLUser *bl);
 #define SetBLUser(x, y)	do { moddata_client(x, blacklist_md).ptr = y; } while(0)
 #define BLUSER(x)	((BLUser *)moddata_client(x, blacklist_md).ptr)
 
-long SNO_BLACKLIST = 0L;
-
 MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, blacklist_config_test);
 
-	CallbackAddEx(modinfo->handle, CALLBACKTYPE_BLACKLIST_CHECK, blacklist_start_check);
+	CallbackAdd(modinfo->handle, CALLBACKTYPE_BLACKLIST_CHECK, blacklist_start_check);
 	return MOD_SUCCESS;
 }
 
@@ -145,13 +147,12 @@ MOD_INIT()
 
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN, 0, blacklist_config_run);
 	HookAdd(modinfo->handle, HOOKTYPE_HANDSHAKE, 0, blacklist_handshake);
+	HookAdd(modinfo->handle, HOOKTYPE_IP_CHANGE, 0, blacklist_ip_change);
 	HookAdd(modinfo->handle, HOOKTYPE_PRE_LOCAL_CONNECT, 0, blacklist_preconnect);
 	HookAdd(modinfo->handle, HOOKTYPE_REHASH, 0, blacklist_rehash);
 	HookAdd(modinfo->handle, HOOKTYPE_REHASH_COMPLETE, 0, blacklist_rehash_complete);
 	HookAdd(modinfo->handle, HOOKTYPE_LOCAL_QUIT, 0, blacklist_quit);
 
-	SnomaskAdd(modinfo->handle, 'b', umode_allow_opers, &SNO_BLACKLIST);
-
 	return MOD_SUCCESS;
 }
 
@@ -251,192 +252,192 @@ int blacklist_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 	if (!ce)
 		return 0;
 	
-	if (strcmp(ce->ce_varname, "blacklist"))
+	if (strcmp(ce->name, "blacklist"))
 		return 0; /* not interested in non-blacklist stuff.. */
 	
-	if (!ce->ce_vardata)
+	if (!ce->value)
 	{
 		config_error("%s:%i: blacklist block without name (use: blacklist somename { })",
-			ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+			ce->file->filename, ce->line_number);
 		*errs = 1;
 		return -1;
 	}
 
 	/* Now actually go parse the blacklist { } block */
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->ce_varname, "dns"))
+		if (!strcmp(cep->name, "dns"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
-				if (!strcmp(cepp->ce_varname, "reply"))
+				if (!strcmp(cepp->name, "reply"))
 				{
 					if (has_dns_reply)
 					{
 						/* this is an error (not a warning) */
 						config_error("%s:%i: blacklist block may contain only one blacklist::dns::reply item. "
 									 "You can specify multiple replies by using: reply { 1; 2; 4; };",
-									 cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+									 cepp->file->filename, cepp->line_number);
 						errors++;
 						continue;
 					}
-					if (!cepp->ce_vardata && !cepp->ce_entries)
+					if (!cepp->value && !cepp->items)
 					{
-						config_error_blank(cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum, "blacklist::dns::reply");
+						config_error_blank(cepp->file->filename, cepp->line_number, "blacklist::dns::reply");
 						errors++;
 						continue;
 					}
 					has_dns_reply = 1; /* we have a reply. now whether it's actually valid is another story.. */
-					if (cepp->ce_vardata && cepp->ce_entries)
+					if (cepp->value && cepp->items)
 					{
 						config_error("%s:%i: blacklist::dns::reply must be either using format 'reply 1;' or "
 									 "'reply { 1; 2; 4; }; but not both formats at the same time.",
-									 cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+									 cepp->file->filename, cepp->line_number);
 						errors++;
 						continue;
 					}
-					if (cepp->ce_vardata)
+					if (cepp->value)
 					{
-						if (atoi(cepp->ce_vardata) <= 0)
+						if (atoi(cepp->value) <= 0)
 						{
 							config_error("%s:%i: blacklist::dns::reply must be >0",
-								cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+								cepp->file->filename, cepp->line_number);
 							errors++;
 							continue;
 						}
 					}
-					if (cepp->ce_entries)
+					if (cepp->items)
 					{
-						for (ceppp = cepp->ce_entries; ceppp; ceppp=ceppp->ce_next)
+						for (ceppp = cepp->items; ceppp; ceppp=ceppp->next)
 						{
-							if (atoi(ceppp->ce_varname) <= 0)
+							if (atoi(ceppp->name) <= 0)
 							{
 								config_error("%s:%i: all items in blacklist::dns::reply must be >0",
-									cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+									cepp->file->filename, cepp->line_number);
 								errors++;
 							}
 						}
 					}
 				} else
-				if (!cepp->ce_vardata)
+				if (!cepp->value)
 				{
-					config_error_empty(cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum,
-						"blacklist::dns", cepp->ce_varname);
+					config_error_empty(cepp->file->filename, cepp->line_number,
+						"blacklist::dns", cepp->name);
 					errors++;
 					continue;
 				} else
-				if (!strcmp(cepp->ce_varname, "name"))
+				if (!strcmp(cepp->name, "name"))
 				{
 					if (has_dns_name)
 					{
-						config_warn_duplicate(cepp->ce_fileptr->cf_filename,
-							cepp->ce_varlinenum, "blacklist::dns::name");
+						config_warn_duplicate(cepp->file->filename,
+							cepp->line_number, "blacklist::dns::name");
 					}
 					has_dns_name = 1;
 				} else
-				if (!strcmp(cepp->ce_varname, "type"))
+				if (!strcmp(cepp->name, "type"))
 				{
 					if (has_dns_type)
 					{
-						config_warn_duplicate(cepp->ce_fileptr->cf_filename,
-							cepp->ce_varlinenum, "blacklist::dns::type");
+						config_warn_duplicate(cepp->file->filename,
+							cepp->line_number, "blacklist::dns::type");
 					}
 					has_dns_type = 1;
-					if (!strcmp(cepp->ce_vardata, "record"))
+					if (!strcmp(cepp->value, "record"))
 						;
-					else if (!strcmp(cepp->ce_vardata, "bitmask"))
+					else if (!strcmp(cepp->value, "bitmask"))
 						;
 					else
 					{
 						config_error("%s:%i: unknown blacklist::dns::type '%s', must be either 'record' or 'bitmask'",
-							cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum, cepp->ce_vardata);
+							cepp->file->filename, cepp->line_number, cepp->value);
 						errors++;
 					}
 				}
 			}
 		} else
-		if (!cep->ce_vardata)
+		if (!cep->value)
 		{
-			config_error_empty(cep->ce_fileptr->cf_filename, cep->ce_varlinenum,
-				"blacklist", cep->ce_varname);
+			config_error_empty(cep->file->filename, cep->line_number,
+				"blacklist", cep->name);
 			errors++;
 			continue;
 		}
-		else if (!strcmp(cep->ce_varname, "action"))
+		else if (!strcmp(cep->name, "action"))
 		{
 			if (has_action)
 			{
-				config_warn_duplicate(cep->ce_fileptr->cf_filename,
-					cep->ce_varlinenum, "blacklist::action");
+				config_warn_duplicate(cep->file->filename,
+					cep->line_number, "blacklist::action");
 				continue;
 			}
 			has_action = 1;
-			if (!banact_stringtoval(cep->ce_vardata))
+			if (!banact_stringtoval(cep->value))
 			{
 				config_error("%s:%i: blacklist::action has unknown action type '%s'",
-					cep->ce_fileptr->cf_filename, cep->ce_varlinenum, cep->ce_vardata);
+					cep->file->filename, cep->line_number, cep->value);
 				errors++;
 			}
 		}
-		else if (!strcmp(cep->ce_varname, "ban-time"))
+		else if (!strcmp(cep->name, "ban-time"))
 		{
 			if (has_ban_time)
 			{
-				config_warn_duplicate(cep->ce_fileptr->cf_filename,
-					cep->ce_varlinenum, "blacklist::ban-time");
+				config_warn_duplicate(cep->file->filename,
+					cep->line_number, "blacklist::ban-time");
 				continue;
 			}
 			has_ban_time = 1;
 		} else
-		if (!strcmp(cep->ce_varname, "reason"))
+		if (!strcmp(cep->name, "reason"))
 		{
 			if (has_reason)
 			{
-				config_warn_duplicate(cep->ce_fileptr->cf_filename,
-					cep->ce_varlinenum, "blacklist::reason");
+				config_warn_duplicate(cep->file->filename,
+					cep->line_number, "blacklist::reason");
 				continue;
 			}
 			has_reason = 1;
 		}
 		else
 		{
-			config_error_unknown(cep->ce_fileptr->cf_filename, cep->ce_varlinenum,
-				"blacklist", cep->ce_varname);
+			config_error_unknown(cep->file->filename, cep->line_number,
+				"blacklist", cep->name);
 			errors++;
 		}
 	}
 
 	if (!has_action)
 	{
-		config_error_missing(ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
+		config_error_missing(ce->file->filename, ce->line_number,
 			"blacklist::action");
 		errors++;
 	}
 
 	if (!has_reason)
 	{
-		config_error_missing(ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
+		config_error_missing(ce->file->filename, ce->line_number,
 			"blacklist::reason");
 		errors++;
 	}
 
 	if (!has_dns_name)
 	{
-		config_error_missing(ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
+		config_error_missing(ce->file->filename, ce->line_number,
 			"blacklist::dns::name");
 		errors++;
 	}
 
 	if (!has_dns_type)
 	{
-		config_error_missing(ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
+		config_error_missing(ce->file->filename, ce->line_number,
 			"blacklist::dns::type");
 		errors++;
 	}
 
 	if (!has_dns_reply)
 	{
-		config_error_missing(ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
+		config_error_missing(ce->file->filename, ce->line_number,
 			"blacklist::dns::reply");
 		errors++;
 	}
@@ -453,12 +454,12 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	if (type != CONFIG_MAIN)
 		return 0;
 	
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "blacklist"))
+	if (!ce || !ce->name || strcmp(ce->name, "blacklist"))
 		return 0; /* not interested */
 
 	d = safe_alloc(sizeof(Blacklist));
-	safe_strdup(d->name, ce->ce_vardata);
-	/* set some defaults. TODO: use set::blacklist or something ? */
+	safe_strdup(d->name, ce->value);
+	/* set some defaults */
 	d->action = BAN_ACT_KILL;
 	safe_strdup(d->reason, "Your IP is on a DNS Blacklist");
 	d->ban_time = 3600;
@@ -468,28 +469,28 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	d->backend = safe_alloc(sizeof(BlacklistBackend));
 	d->backend->dns = safe_alloc(sizeof(DNSBL));
 
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->ce_varname, "dns"))
+		if (!strcmp(cep->name, "dns"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
-				if (!strcmp(cepp->ce_varname, "reply"))
+				if (!strcmp(cepp->name, "reply"))
 				{
-					if (cepp->ce_vardata)
+					if (cepp->value)
 					{
 						/* single reply */
 						d->backend->dns->reply = safe_alloc(sizeof(int)*2);
-						d->backend->dns->reply[0] = atoi(cepp->ce_vardata);
+						d->backend->dns->reply[0] = atoi(cepp->value);
 						d->backend->dns->reply[1] = 0;
 					} else
-					if (cepp->ce_entries)
+					if (cepp->items)
 					{
 						/* (potentially) multiple reply values */
 						int cnt = 0;
-						for (ceppp = cepp->ce_entries; ceppp; ceppp = ceppp->ce_next)
+						for (ceppp = cepp->items; ceppp; ceppp = ceppp->next)
 						{
-							if (ceppp->ce_varname)
+							if (ceppp->name)
 								cnt++;
 						}
 						
@@ -499,37 +500,37 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 						d->backend->dns->reply = safe_alloc(sizeof(int)*(cnt+1));
 						
 						cnt = 0;
-						for (ceppp = cepp->ce_entries; ceppp; ceppp = ceppp->ce_next)
+						for (ceppp = cepp->items; ceppp; ceppp = ceppp->next)
 						{
-							d->backend->dns->reply[cnt++] = atoi(ceppp->ce_varname);
+							d->backend->dns->reply[cnt++] = atoi(ceppp->name);
 						}
 						d->backend->dns->reply[cnt] = 0;
 					}
 				} else
-				if (!strcmp(cepp->ce_varname, "type"))
+				if (!strcmp(cepp->name, "type"))
 				{
-					if (!strcmp(cepp->ce_vardata, "record"))
+					if (!strcmp(cepp->value, "record"))
 						d->backend->dns->type = DNSBL_RECORD;
-					else if (!strcmp(cepp->ce_vardata, "bitmask"))
+					else if (!strcmp(cepp->value, "bitmask"))
 						d->backend->dns->type = DNSBL_BITMASK;
 				} else
-				if (!strcmp(cepp->ce_varname, "name"))
+				if (!strcmp(cepp->name, "name"))
 				{
-					safe_strdup(d->backend->dns->name, cepp->ce_vardata);
+					safe_strdup(d->backend->dns->name, cepp->value);
 				}
 			}
 		}
-		else if (!strcmp(cep->ce_varname, "action"))
+		else if (!strcmp(cep->name, "action"))
 		{
-			d->action = banact_stringtoval(cep->ce_vardata);
+			d->action = banact_stringtoval(cep->value);
 		}
-		else if (!strcmp(cep->ce_varname, "reason"))
+		else if (!strcmp(cep->name, "reason"))
 		{
-			safe_strdup(d->reason, cep->ce_vardata);
+			safe_strdup(d->reason, cep->value);
 		}
-		else if (!strcmp(cep->ce_varname, "ban-time"))
+		else if (!strcmp(cep->name, "ban-time"))
 		{
-			d->ban_time = config_checkval(cep->ce_vardata, CFG_TIME);
+			d->ban_time = config_checkval(cep->value, CFG_TIME);
 		}
 	}
 
@@ -554,6 +555,12 @@ int blacklist_handshake(Client *client)
 	return 0;
 }
 
+int blacklist_ip_change(Client *client, const char *oldip)
+{
+	blacklist_start_check(client);
+	return 0;
+}
+
 int blacklist_start_check(Client *client)
 {
 	Blacklist *bl;
@@ -644,7 +651,7 @@ void blacklist_cancel(BLUser *bl)
 	bl->client = NULL;
 }
 
-int blacklist_quit(Client *client, MessageTag *mtags, char *comment)
+int blacklist_quit(Client *client, MessageTag *mtags, const char *comment)
 {
 	if (BLUSER(client))
 		blacklist_cancel(BLUSER(client));
@@ -725,10 +732,17 @@ int blacklist_parse_reply(struct hostent *he, int entry)
  * from blacklist_preconnect() for softbans that need to be delayed
  * as to give the user the opportunity to do SASL Authentication.
  */
-int blacklist_action(Client *client, char *opernotice, BanAction ban_action, char *ban_reason, long ban_time)
+int blacklist_action(Client *client, char *opernotice, BanAction ban_action, char *ban_reason, long ban_time,
+                     char *blacklist, char *blacklist_dns_name, int blacklist_dns_reply)
 {
-	sendto_snomask(SNO_BLACKLIST, "%s", opernotice);
-	ircd_log(LOG_KILL, "%s", opernotice);
+	unreal_log_raw(ULOG_INFO, "blacklist", "BLACKLIST_HIT", client,
+	               opernotice,
+	               log_data_string("blacklist_name", blacklist),
+	               log_data_string("blacklist_dns_name", blacklist_dns_name),
+	               log_data_integer("blacklist_dns_reply", blacklist_dns_reply),
+	               log_data_string("ban_action", banact_valtostring(ban_action)),
+	               log_data_string("ban_reason", ban_reason),
+	               log_data_integer("ban_time", ban_time));
 	if (ban_action == BAN_ACT_WARN)
 		return 0;
 	return place_host_ban(client, ban_action, ban_reason, ban_time);
@@ -766,9 +780,12 @@ void blacklist_hit(Client *client, Blacklist *bl, int reply)
 		blu->save_tkltime = bl->ban_time;
 		safe_strdup(blu->save_opernotice, opernotice);
 		safe_strdup(blu->save_reason, banbuf);
+		safe_strdup(blu->save_blacklist, bl->name);
+		safe_strdup(blu->save_blacklist_dns_name, bl->backend->dns->name);
+		blu->save_blacklist_dns_reply = reply;
 	} else {
 		/* Otherwise, execute the action immediately */
-		blacklist_action(client, opernotice, bl->action, banbuf, bl->ban_time);
+		blacklist_action(client, opernotice, bl->action, banbuf, bl->ban_time, bl->name, bl->backend->dns->name, reply);
 	}
 }
 
@@ -841,7 +858,10 @@ int blacklist_preconnect(Client *client)
 	if (IsLoggedIn(client))
 		return HOOK_CONTINUE; /* yup, so the softban does not apply. */
 
-	if (blacklist_action(client, blu->save_opernotice, blu->save_action, blu->save_reason, blu->save_tkltime))
+	if (blacklist_action(client, blu->save_opernotice, blu->save_action, blu->save_reason, blu->save_tkltime,
+	                     blu->save_blacklist, blu->save_blacklist_dns_name, blu->save_blacklist_dns_reply))
+	{
 		return HOOK_DENY;
+	}
 	return HOOK_CONTINUE; /* exempt */
 }
