@@ -6,24 +6,24 @@
 #include "unrealircd.h"
 
 
-ModuleHeader MOD_HEADER(censor)
+ModuleHeader MOD_HEADER
   = {
 	"chanmodes/censor",
 	"4.2",
 	"Channel Mode +G",
-	"3.2-b8-1",
-	NULL,
+	"UnrealIRCd Team",
+	"unrealircd-6",
     };
 
 
 Cmode_t EXTMODE_CENSOR = 0L;
 
-#define IsCensored(x) ((x)->mode.extmode & EXTMODE_CENSOR)
-
-char *censor_pre_chanmsg(aClient *sptr, aChannel *chptr, char *text, int notice);
-char *censor_pre_local_part(aClient *sptr, aChannel *chptr, char *text);
-char *censor_pre_local_quit(aClient *sptr, char *text);
+#define IsCensored(x) ((x)->mode.mode & EXTMODE_CENSOR)
 
+int censor_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype);
+const char *censor_pre_local_part(Client *client, Channel *channel, const char *text);
+const char *censor_pre_local_quit(Client *client, const char *text);
+int censor_stats_badwords_channel(Client *client, const char *para);
 int censor_config_test(ConfigFile *, ConfigEntry *, int, int *);
 int censor_config_run(ConfigFile *, ConfigEntry *, int);
 
@@ -32,13 +32,13 @@ ModuleInfo *ModInfo = NULL;
 ConfigItem_badword *conf_badword_channel = NULL;
 
 
-MOD_TEST(censor)
+MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, censor_config_test);
 	return MOD_SUCCESS;
 }
 	
-MOD_INIT(censor)
+MOD_INIT()
 {
 	CmodeInfo req;
 
@@ -49,24 +49,24 @@ MOD_INIT(censor)
 	memset(&req, 0, sizeof(req));
 	req.paracount = 0;
 	req.is_ok = extcmode_default_requirechop;
-	req.flag = 'G';
+	req.letter = 'G';
 	CmodeAdd(modinfo->handle, req, &EXTMODE_CENSOR);
 
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_CHANMSG, 0, censor_pre_chanmsg);
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_LOCAL_PART, 0, censor_pre_local_part);
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_LOCAL_QUIT, 0, censor_pre_local_quit);
-	
+	HookAdd(modinfo->handle, HOOKTYPE_CAN_SEND_TO_CHANNEL, 0, censor_can_send_to_channel);
+	HookAddConstString(modinfo->handle, HOOKTYPE_PRE_LOCAL_PART, 0, censor_pre_local_part);
+	HookAddConstString(modinfo->handle, HOOKTYPE_PRE_LOCAL_QUIT, 0, censor_pre_local_quit);
+	HookAdd(modinfo->handle, HOOKTYPE_STATS, 0, censor_stats_badwords_channel);
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN, 0, censor_config_run);
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(censor)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
 
-MOD_UNLOAD(censor)
+MOD_UNLOAD()
 {
 	ConfigItem_badword *badword, *next;
 
@@ -88,99 +88,99 @@ int censor_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 	if (type != CONFIG_MAIN)
 		return 0;
 	
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "badword"))
+	if (!ce || !ce->name || strcmp(ce->name, "badword"))
 		return 0; /* not interested */
 
-	if (!ce->ce_vardata)
+	if (!ce->value)
 	{
 		config_error("%s:%i: badword without type",
-			ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+			ce->file->filename, ce->line_number);
 		return 1;
 	}
-	else if (strcmp(ce->ce_vardata, "channel") && 
-	         strcmp(ce->ce_vardata, "quit") && strcmp(ce->ce_vardata, "all")) {
+	else if (strcmp(ce->value, "channel") && 
+	         strcmp(ce->value, "quit") && strcmp(ce->value, "all")) {
 /*			config_error("%s:%i: badword with unknown type",
-				ce->ce_fileptr->cf_filename, ce->ce_varlinenum); -- can't do that.. */
+				ce->file->filename, ce->line_number); -- can't do that.. */
 		return 0; /* unhandled */
 	}
 	
-	if (!strcmp(ce->ce_vardata, "quit"))
+	if (!strcmp(ce->value, "quit"))
 	{
 		config_error("%s:%i: badword quit has been removed. We just use the bad words from "
 		             "badword channel { } instead.",
-		             ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+		             ce->file->filename, ce->line_number);
 		return 0; /* pretend unhandled.. ok not just pretend.. ;) */
 	}
 
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
 		if (config_is_blankorempty(cep, "badword"))
 		{
 			errors++;
 			continue;
 		}
-		if (!strcmp(cep->ce_varname, "word"))
+		if (!strcmp(cep->name, "word"))
 		{
-			char *errbuf;
+			const char *errbuf;
 			if (has_word)
 			{
-				config_warn_duplicate(cep->ce_fileptr->cf_filename, 
-					cep->ce_varlinenum, "badword::word");
+				config_warn_duplicate(cep->file->filename, 
+					cep->line_number, "badword::word");
 				continue;
 			}
 			has_word = 1;
-			if ((errbuf = badword_config_check_regex(cep->ce_vardata,1,1)))
+			if ((errbuf = badword_config_check_regex(cep->value,1,1)))
 			{
 				config_error("%s:%i: badword::%s contains an invalid regex: %s",
-					cep->ce_fileptr->cf_filename,
-					cep->ce_varlinenum,
-					cep->ce_varname, errbuf);
+					cep->file->filename,
+					cep->line_number,
+					cep->name, errbuf);
 				errors++;
 			}
 		}
-		else if (!strcmp(cep->ce_varname, "replace"))
+		else if (!strcmp(cep->name, "replace"))
 		{
 			if (has_replace)
 			{
-				config_warn_duplicate(cep->ce_fileptr->cf_filename, 
-					cep->ce_varlinenum, "badword::replace");
+				config_warn_duplicate(cep->file->filename, 
+					cep->line_number, "badword::replace");
 				continue;
 			}
 			has_replace = 1;
 		}
-		else if (!strcmp(cep->ce_varname, "action"))
+		else if (!strcmp(cep->name, "action"))
 		{
 			if (has_action)
 			{
-				config_warn_duplicate(cep->ce_fileptr->cf_filename, 
-					cep->ce_varlinenum, "badword::action");
+				config_warn_duplicate(cep->file->filename, 
+					cep->line_number, "badword::action");
 				continue;
 			}
 			has_action = 1;
-			if (!strcmp(cep->ce_vardata, "replace"))
+			if (!strcmp(cep->value, "replace"))
 				action = 'r';
-			else if (!strcmp(cep->ce_vardata, "block"))
+			else if (!strcmp(cep->value, "block"))
 				action = 'b';
 			else
 			{
 				config_error("%s:%d: Unknown badword::action '%s'",
-					cep->ce_fileptr->cf_filename, cep->ce_varlinenum,
-					cep->ce_vardata);
+					cep->file->filename, cep->line_number,
+					cep->value);
 				errors++;
 			}
 				
 		}
 		else
 		{
-			config_error_unknown(cep->ce_fileptr->cf_filename, cep->ce_varlinenum,
-				"badword", cep->ce_varname);
+			config_error_unknown(cep->file->filename, cep->line_number,
+				"badword", cep->name);
 			errors++;
 		}
 	}
 
 	if (!has_word)
 	{
-		config_error_missing(ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
+		config_error_missing(ce->file->filename, ce->line_number,
 			"badword::word");
 		errors++;
 	}
@@ -189,7 +189,7 @@ int censor_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 		if (has_replace && action == 'b')
 		{
 			config_error("%s:%i: badword::action is block but badword::replace exists",
-				ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+				ce->file->filename, ce->line_number);
 			errors++;
 		}
 	}
@@ -207,39 +207,39 @@ int censor_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	if (type != CONFIG_MAIN)
 		return 0;
 	
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "badword"))
+	if (!ce || !ce->name || strcmp(ce->name, "badword"))
 		return 0; /* not interested */
 
-	if (strcmp(ce->ce_vardata, "channel") && strcmp(ce->ce_vardata, "all"))
+	if (strcmp(ce->value, "channel") && strcmp(ce->value, "all"))
 	        return 0; /* not for us */
 
-	ca = MyMallocEx(sizeof(ConfigItem_badword));
+	ca = safe_alloc(sizeof(ConfigItem_badword));
 	ca->action = BADWORD_REPLACE;
 
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->ce_varname, "action"))
+		if (!strcmp(cep->name, "action"))
 		{
-			if (!strcmp(cep->ce_vardata, "block"))
+			if (!strcmp(cep->value, "block"))
 			{
 				ca->action = BADWORD_BLOCK;
 			}
 		}
-		else if (!strcmp(cep->ce_varname, "replace"))
+		else if (!strcmp(cep->name, "replace"))
 		{
-			safestrdup(ca->replace, cep->ce_vardata);
+			safe_strdup(ca->replace, cep->value);
 		} else
-		if (!strcmp(cep->ce_varname, "word"))
+		if (!strcmp(cep->name, "word"))
 		{
 			word = cep;
 		}
 	}
 
-	badword_config_process(ca, word->ce_vardata);
+	badword_config_process(ca, word->value);
 
-	if (!strcmp(ce->ce_vardata, "channel"))
+	if (!strcmp(ce->value, "channel"))
 		AddListItem(ca, conf_badword_channel);
-	else if (!strcmp(ce->ce_vardata, "all"))
+	else if (!strcmp(ce->value, "all"))
 	{
 		AddListItem(ca, conf_badword_channel);
 		return 0; /* pretend we didn't see it, so other modules can handle 'all' as well */
@@ -248,50 +248,47 @@ int censor_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	return 1;
 }
 
-char *stripbadwords_channel(char *str, int *blocked)
+const char *stripbadwords_channel(const char *str, int *blocked)
 {
 	return stripbadwords(str, conf_badword_channel, blocked);
 }
 
-char *censor_pre_chanmsg(aClient *sptr, aChannel *chptr, char *text, int notice)
+int censor_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype)
 {
 	int blocked;
 	Hook *h;
 	int i;
 
-	if (!IsCensored(chptr))
-		return text;
+	if (!IsCensored(channel))
+		return HOOK_CONTINUE;
 
 	for (h = Hooks[HOOKTYPE_CAN_BYPASS_CHANNEL_MESSAGE_RESTRICTION]; h; h = h->next)
 	{
-		i = (*(h->func.intfunc))(sptr, chptr, BYPASS_CHANMSG_CENSOR);
+		i = (*(h->func.intfunc))(client, channel, BYPASS_CHANMSG_CENSOR);
+		if (i == HOOK_ALLOW)
+			return HOOK_CONTINUE; /* bypass censor restriction */
 		if (i != HOOK_CONTINUE)
 			break;
 	}
-	if (i == HOOK_ALLOW)
-		return text; /* bypass */
 
-	text = stripbadwords_channel(text, &blocked);
+	*msg = stripbadwords_channel(*msg, &blocked);
 	if (blocked)
 	{
-		if (!notice)
-			sendto_one(sptr, err_str(ERR_CANNOTSENDTOCHAN),
-				me.name, sptr->name, chptr->chname,
-				"Swearing is not permitted in this channel", chptr->chname);
-		return NULL;
+		*errmsg = "Swearing is not permitted in this channel";
+		return HOOK_DENY;
 	}
 
-	return text;
+	return HOOK_CONTINUE;
 }
 
-char *censor_pre_local_part(aClient *sptr, aChannel *chptr, char *text)
+const char *censor_pre_local_part(Client *client, Channel *channel, const char *text)
 {
 	int blocked;
 
 	if (!text)
 		return NULL;
 
-	if (!IsCensored(chptr))
+	if (!IsCensored(channel))
 		return text;
 
 	text = stripbadwords_channel(text, &blocked);
@@ -299,41 +296,42 @@ char *censor_pre_local_part(aClient *sptr, aChannel *chptr, char *text)
 }
 
 /** Is any channel where the user is in +G? */
-static int IsAnyChannelCensored(aClient *sptr)
+static int IsAnyChannelCensored(Client *client)
 {
 	Membership *lp;
 
-	for (lp = sptr->user->channel; lp; lp = lp->next)
-		if (IsCensored(lp->chptr))
+	for (lp = client->user->channel; lp; lp = lp->next)
+		if (IsCensored(lp->channel))
 			return 1;
 	return 0;
 }
 
-char *censor_pre_local_quit(aClient *sptr, char *text)
+const char *censor_pre_local_quit(Client *client, const char *text)
 {
 	int blocked = 0;
 
 	if (!text)
 		return NULL;
 
-	if (IsAnyChannelCensored(sptr))
+	if (IsAnyChannelCensored(client))
 		text = stripbadwords_channel(text, &blocked);
 
 	return blocked ? NULL : text;
 }
 
-// TODO: when stats is modular, make it call this for badwords
-int stats_badwords(aClient *sptr, char *para)
+int censor_stats_badwords_channel(Client *client, const char *para)
 {
 	ConfigItem_badword *words;
 
+	if (!para || !(!strcmp(para, "b") || !strcasecmp(para, "badword")))
+		return 0;
+
 	for (words = conf_badword_channel; words; words = words->next)
 	{
-		sendto_one(sptr, ":%s %i %s :c %c %s%s%s %s",
-		           me.name, RPL_TEXT, sptr->name, words->type & BADW_TYPE_REGEX ? 'R' : 'F',
+		sendtxtnumeric(client, "c %c %s%s%s %s", words->type & BADW_TYPE_REGEX ? 'R' : 'F',
 		           (words->type & BADW_TYPE_FAST_L) ? "*" : "", words->word,
 		           (words->type & BADW_TYPE_FAST_R) ? "*" : "",
 		           words->action == BADWORD_REPLACE ? (words->replace ? words->replace : "<censored>") : "");
 	}
-	return 0;
+	return 1;
 }
