@@ -19,7 +19,7 @@ ModuleHeader MOD_HEADER
 	CONNTHROTTLE_VERSION,
 	"Connection throttler - by Syzop",
 	"UnrealIRCd Team",
-	"unrealircd-5",
+	"unrealircd-6",
     };
 
 typedef struct {
@@ -146,9 +146,6 @@ int ct_config_posttest(int *errs)
 	return errors ? -1 : 1;
 }
 
-#ifndef CheckNull
- #define CheckNull(x) if ((!(x)->ce_vardata) || (!(*((x)->ce_vardata)))) { config_error("%s:%i: missing parameter", (x)->ce_fileptr->cf_filename, (x)->ce_varlinenum); errors++; continue; }
-#endif
 /** Test the set::connthrottle configuration */
 int ct_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 {
@@ -159,113 +156,113 @@ int ct_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 		return 0;
 	
 	/* We are only interrested in set::connthrottle.. */
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "connthrottle"))
+	if (!ce || !ce->name || strcmp(ce->name, "connthrottle"))
 		return 0;
 	
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->ce_varname, "known-users"))
+		if (!strcmp(cep->name, "known-users"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
 				CheckNull(cepp);
-				if (!strcmp(cepp->ce_varname, "minimum-reputation-score"))
+				if (!strcmp(cepp->name, "minimum-reputation-score"))
 				{
-					int cnt = atoi(cepp->ce_vardata);
+					int cnt = atoi(cepp->value);
 					if (cnt < 1)
 					{
 						config_error("%s:%i: set::connthrottle::known-users::minimum-reputation-score should be at least 1",
-							cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+							cepp->file->filename, cepp->line_number);
 						errors++;
 						continue;
 					}
 				} else
-				if (!strcmp(cepp->ce_varname, "sasl-bypass"))
+				if (!strcmp(cepp->name, "sasl-bypass"))
 				{
 				} else
-				if (!strcmp(cepp->ce_varname, "webirc-bypass"))
+				if (!strcmp(cepp->name, "webirc-bypass"))
 				{
 				} else
 				{
-					config_error_unknown(cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum,
-					                     "set::connthrottle::known-users", cepp->ce_varname);
+					config_error_unknown(cepp->file->filename, cepp->line_number,
+					                     "set::connthrottle::known-users", cepp->name);
 					errors++;
 				}
 			}
 		} else
-		if (!strcmp(cep->ce_varname, "new-users"))
+		if (!strcmp(cep->name, "new-users"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
 				CheckNull(cepp);
-				if (!strcmp(cepp->ce_varname, "local-throttle"))
+				if (!strcmp(cepp->name, "local-throttle"))
 				{
 					int cnt, period;
-					if (!config_parse_flood(cepp->ce_vardata, &cnt, &period) ||
+					if (!config_parse_flood(cepp->value, &cnt, &period) ||
 					    (cnt < 1) || (cnt > 2000000000) || (period > 2000000000))
 					{
 						config_error("%s:%i: set::connthrottle::new-users::local-throttle error. "
 							     "Syntax is <count>:<period> (eg 6:60), "
 							     "and count and period should be non-zero.",
-							     cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+							     cepp->file->filename, cepp->line_number);
 						errors++;
 						continue;
 					}
 				} else
-				if (!strcmp(cepp->ce_varname, "global-throttle"))
+				if (!strcmp(cepp->name, "global-throttle"))
 				{
 					int cnt, period;
-					if (!config_parse_flood(cepp->ce_vardata, &cnt, &period) ||
+					if (!config_parse_flood(cepp->value, &cnt, &period) ||
 					    (cnt < 1) || (cnt > 2000000000) || (period > 2000000000))
 					{
 						config_error("%s:%i: set::connthrottle::new-users::global-throttle error. "
 							     "Syntax is <count>:<period> (eg 6:60), "
 							     "and count and period should be non-zero.",
-							     cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+							     cepp->file->filename, cepp->line_number);
 						errors++;
 						continue;
 					}
 				} else
 				{
-					config_error_unknown(cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum,
-					                     "set::connthrottle::new-users", cepp->ce_varname);
+					config_error_unknown(cepp->file->filename, cepp->line_number,
+					                     "set::connthrottle::new-users", cepp->name);
 					errors++;
 				}
 			}
 		} else
-		if (!strcmp(cep->ce_varname, "disabled-when"))
+		if (!strcmp(cep->name, "disabled-when"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
 				CheckNull(cepp);
-				if (!strcmp(cepp->ce_varname, "start-delay"))
+				if (!strcmp(cepp->name, "start-delay"))
 				{
-					int cnt = config_checkval(cepp->ce_vardata, CFG_TIME);
+					int cnt = config_checkval(cepp->value, CFG_TIME);
 					if ((cnt < 0) || (cnt > 3600))
 					{
 						config_error("%s:%i: set::connthrottle::disabled-when::start-delay should be in range 0-3600",
-							cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum);
+							cepp->file->filename, cepp->line_number);
 						errors++;
 						continue;
 					}
 				} else
-				if (!strcmp(cepp->ce_varname, "reputation-gathering"))
+				if (!strcmp(cepp->name, "reputation-gathering"))
 				{
 				} else
 				{
-					config_error_unknown(cepp->ce_fileptr->cf_filename, cepp->ce_varlinenum,
-					                     "set::connthrottle::disabled-when", cepp->ce_varname);
+					config_error_unknown(cepp->file->filename, cepp->line_number,
+					                     "set::connthrottle::disabled-when", cepp->name);
 					errors++;
 				}
 			}
 		} else
-		if (!strcmp(cep->ce_varname, "reason"))
+		if (!strcmp(cep->name, "reason"))
 		{
 			CheckNull(cep);
 		} else
 		{
 			config_error("%s:%i: unknown directive set::connthrottle::%s",
-				cep->ce_fileptr->cf_filename, cep->ce_varlinenum, cep->ce_varname);
+				cep->file->filename, cep->line_number, cep->name);
 			errors++;
 			continue;
 		}
@@ -284,48 +281,48 @@ int ct_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		return 0;
 	
 	/* We are only interrested in set::connthrottle.. */
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "connthrottle"))
+	if (!ce || !ce->name || strcmp(ce->name, "connthrottle"))
 		return 0;
 	
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->ce_varname, "known-users"))
+		if (!strcmp(cep->name, "known-users"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
-				if (!strcmp(cepp->ce_varname, "minimum-reputation-score"))
-					cfg.minimum_reputation_score = atoi(cepp->ce_vardata);
-				else if (!strcmp(cepp->ce_varname, "sasl-bypass"))
-					cfg.sasl_bypass = config_checkval(cepp->ce_vardata, CFG_YESNO);
-				else if (!strcmp(cepp->ce_varname, "webirc-bypass"))
-					cfg.webirc_bypass = config_checkval(cepp->ce_vardata, CFG_YESNO);
+				if (!strcmp(cepp->name, "minimum-reputation-score"))
+					cfg.minimum_reputation_score = atoi(cepp->value);
+				else if (!strcmp(cepp->name, "sasl-bypass"))
+					cfg.sasl_bypass = config_checkval(cepp->value, CFG_YESNO);
+				else if (!strcmp(cepp->name, "webirc-bypass"))
+					cfg.webirc_bypass = config_checkval(cepp->value, CFG_YESNO);
 			}
 		} else
-		if (!strcmp(cep->ce_varname, "new-users"))
+		if (!strcmp(cep->name, "new-users"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
-				if (!strcmp(cepp->ce_varname, "local-throttle"))
-					config_parse_flood(cepp->ce_vardata, &cfg.local.count, &cfg.local.period);
-				else if (!strcmp(cepp->ce_varname, "global-throttle"))
-					config_parse_flood(cepp->ce_vardata, &cfg.global.count, &cfg.global.period);
+				if (!strcmp(cepp->name, "local-throttle"))
+					config_parse_flood(cepp->value, &cfg.local.count, &cfg.local.period);
+				else if (!strcmp(cepp->name, "global-throttle"))
+					config_parse_flood(cepp->value, &cfg.global.count, &cfg.global.period);
 			}
 		} else
-		if (!strcmp(cep->ce_varname, "disabled-when"))
+		if (!strcmp(cep->name, "disabled-when"))
 		{
-			for (cepp = cep->ce_entries; cepp; cepp = cepp->ce_next)
+			for (cepp = cep->items; cepp; cepp = cepp->next)
 			{
-				if (!strcmp(cepp->ce_varname, "start-delay"))
-					cfg.start_delay = config_checkval(cepp->ce_vardata, CFG_TIME);
-				else if (!strcmp(cepp->ce_varname, "reputation-gathering"))
-					cfg.reputation_gathering = config_checkval(cepp->ce_vardata, CFG_TIME);
+				if (!strcmp(cepp->name, "start-delay"))
+					cfg.start_delay = config_checkval(cepp->value, CFG_TIME);
+				else if (!strcmp(cepp->name, "reputation-gathering"))
+					cfg.reputation_gathering = config_checkval(cepp->value, CFG_TIME);
 			}
 		} else
-		if (!strcmp(cep->ce_varname, "reason"))
+		if (!strcmp(cep->name, "reason"))
 		{
 			safe_free(cfg.reason);
-			cfg.reason = safe_alloc(strlen(cep->ce_vardata)+16);
-			sprintf(cfg.reason, "Throttled: %s", cep->ce_vardata);
+			cfg.reason = safe_alloc(strlen(cep->value)+16);
+			sprintf(cfg.reason, "Throttled: %s", cep->value);
 		}
 	}
 	return 1;
@@ -360,16 +357,18 @@ EVENT(connthrottle_evt)
 
 	if (ucounter->rejected_clients)
 	{
-		snprintf(buf, sizeof(buf),
-		         "[ConnThrottle] Stats for this server past 60 secs: Connections rejected: %d. Accepted: %d known user(s), %d SASL, %d WEBIRC and %d new user(s).",
-		         ucounter->rejected_clients,
-		         ucounter->allowed_score,
-		         ucounter->allowed_sasl,
-			 ucounter->allowed_webirc,
-		         ucounter->allowed_other);
-
-		sendto_realops("%s", buf);
-		ircd_log(LOG_ERROR, "%s", buf);
+		unreal_log(ULOG_INFO, "connthrottle", "CONNTHROTLE_REPORT", NULL,
+		           "ConnThrottle] Stats for this server past 60 secs: "
+		           "Connections rejected: $num_rejected. "
+		           "Accepted: $num_accepted_known_users known user(s), "
+		           "$num_accepted_sasl SASL, "
+		           "$num_accepted_webirc WEBIRC and "
+		           "$num_accepted_unknown_users new user(s).",
+		           log_data_integer("num_rejected", ucounter->rejected_clients),
+		           log_data_integer("num_accepted_known_users", ucounter->allowed_score),
+		           log_data_integer("num_accepted_sasl", ucounter->allowed_sasl),
+		           log_data_integer("num_accepted_webirc", ucounter->allowed_webirc),
+		           log_data_integer("num_accepted_unknown_users", ucounter->allowed_other));
 	}
 
 	/* Reset stats for next message */
@@ -391,7 +390,7 @@ int ct_pre_lconnect(Client *client)
 	int throttle=0;
 	int score;
 
-	if (me.local->firsttime + cfg.start_delay > TStime())
+	if (me.local->creationtime + cfg.start_delay > TStime())
 		return HOOK_CONTINUE; /* no throttle: start delay */
 
 	if (ucounter->disabled)
@@ -436,10 +435,10 @@ int ct_pre_lconnect(Client *client)
 		/* We send the LARGE banner if throttling was activated */
 		if (!ucounter->throttling_previous_minute && !ucounter->throttling_banner_displayed)
 		{
-			ircd_log(LOG_ERROR, "[ConnThrottle] Connection throttling has been ACTIVATED due to a HIGH CONNECTION RATE.");
-			sendto_realops("[ConnThrottle] Connection throttling has been ACTIVATED due to a HIGH CONNECTION RATE.");
-			sendto_realops("[ConnThrottle] Users with IP addresses that have not been seen before will be rejected above the set connection rate. Known users can still get in.");
-			sendto_realops("[ConnThrottle] For more information see https://www.unrealircd.org/docs/ConnThrottle");
+			unreal_log(ULOG_WARNING, "connthrottle", "CONNTHROTLE_ACTIVATED", NULL,
+			           "[ConnThrottle] Connection throttling has been ACTIVATED due to a HIGH CONNECTION RATE.\n"
+			           "Users with IP addresses that have not been seen before will be rejected above the set connection rate. Known users can still get in.\n"
+			           "or more information see https://www.unrealircd.org/docs/ConnThrottle");
 			ucounter->throttling_banner_displayed = 1;
 		}
 		exit_client(client, NULL, cfg.reason);
@@ -478,7 +477,7 @@ int ct_lconnect(Client *client)
 {
 	int score;
 
-	if (me.local->firsttime + cfg.start_delay > TStime())
+	if (me.local->creationtime + cfg.start_delay > TStime())
 		return 0; /* no throttle: start delay */
 
 	if (ucounter->disabled)
@@ -521,7 +520,7 @@ int ct_rconnect(Client *client)
 {
 	int score;
 
-	if (client->srvptr && !IsSynched(client->srvptr))
+	if (client->uplink && !IsSynched(client->uplink))
 		return 0; /* Netmerge: skip */
 
 	if (IsULine(client))
@@ -533,8 +532,8 @@ int ct_rconnect(Client *client)
 	 * set::disabled-when::start-delay restriction on remote
 	 * servers as well.
 	 */
-	if (client->srvptr && client->srvptr->serv && client->srvptr->serv->boottime &&
-	    (TStime() - client->srvptr->serv->boottime < cfg.start_delay))
+	if (client->uplink && client->uplink->server && client->uplink->server->boottime &&
+	    (TStime() - client->uplink->server->boottime < cfg.start_delay))
 	{
 		return 0;
 	}
@@ -584,10 +583,10 @@ CMD_FUNC(ct_throttle)
 			{
 				sendnotice(client, "Module DISABLED because the 'reputation' module has not gathered enough data yet (set::connthrottle::disabled-when::reputation-gathering).");
 			} else
-			if (me.local->firsttime + cfg.start_delay > TStime())
+			if (me.local->creationtime + cfg.start_delay > TStime())
 			{
 				sendnotice(client, "Module DISABLED due to start-delay (set::connthrottle::disabled-when::start-delay), will be enabled in %lld second(s).",
-					(long long)((me.local->firsttime + cfg.start_delay) - TStime()));
+					(long long)((me.local->creationtime + cfg.start_delay) - TStime()));
 			} else
 			{
 				sendnotice(client, "Module ENABLED");
@@ -602,8 +601,8 @@ CMD_FUNC(ct_throttle)
 			return;
 		}
 		ucounter->disabled = 1;
-		sendto_realops("[connthrottle] %s (%s@%s) DISABLED the connthrottle module.",
-			client->name, client->user->username, client->user->realhost);
+		unreal_log(ULOG_WARNING, "connthrottle", "CONNTHROTLE_MODULE_DISABLED", client,
+			   "[ConnThrottle] $client.details DISABLED the connthrottle module.");
 	} else
 	if (!strcasecmp(parv[1], "ON"))
 	{
@@ -612,15 +611,15 @@ CMD_FUNC(ct_throttle)
 			sendnotice(client, "Already ON");
 			return;
 		}
-		sendto_realops("[connthrottle] %s (%s@%s) ENABLED the connthrottle module.",
-			client->name, client->user->username, client->user->realhost);
+		unreal_log(ULOG_WARNING, "connthrottle", "CONNTHROTLE_MODULE_ENABLED", client,
+			   "[ConnThrottle] $client.details ENABLED the connthrottle module.");
 		ucounter->disabled = 0;
 	} else
 	if (!strcasecmp(parv[1], "RESET"))
 	{
 		memset(ucounter, 0, sizeof(UCounter));
-		sendto_realops("[connthrottle] %s (%s@%s) did a RESET on the stats/counters!!",
-			client->name, client->user->username, client->user->realhost);
+		unreal_log(ULOG_WARNING, "connthrottle", "CONNTHROTLE_RESET", client,
+			   "[ConnThrottle] $client.details did a RESET on the statistics/counters.");
 	} else
 	{
 		sendnotice(client, "Unknown option '%s'", parv[1]);
