@@ -23,8 +23,6 @@
    
 #include "unrealircd.h"
 
-#define TIMEDBAN_VERSION "v1.0"
-
 /* Maximum time (in minutes) for a ban */
 #define TIMEDBAN_MAX_TIME	9999
 
@@ -47,43 +45,44 @@
  */
 #define TIMEDBAN_TIMER_DELTA ((TIMEDBAN_TIMER_ITERATION_SPLIT*TIMEDBAN_TIMER)/2)
 
-ModuleHeader MOD_HEADER(timedban)
+ModuleHeader MOD_HEADER
   = {
-	"timedban",
-	TIMEDBAN_VERSION,
+	"extbans/timedban",
+	"1.0",
 	"ExtBan ~t: automatically removed timed bans",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-6",
     };
 
 /* Forward declarations */
-char *timedban_extban_conv_param(char *para_in);
-int timedban_extban_is_ok(aClient* sptr, aChannel* chptr, char* para_in, int checkt, int what, int what2);
-int timedban_is_banned(aClient *sptr, aChannel *chptr, char *ban, int chktype);
-void add_send_mode_param(aChannel *chptr, aClient *from, char what, char mode, char *param);
-char *timedban_chanmsg(aClient *, aClient *, aChannel *, char *, int);
+const char *timedban_extban_conv_param(BanContext *b, Extban *extban);
+int timedban_extban_is_ok(BanContext *b);
+int timedban_is_banned(BanContext *b);
+void add_send_mode_param(Channel *channel, Client *from, char what, char mode, char *param);
+char *timedban_chanmsg(Client *, Client *, Channel *, char *, int);
 
 EVENT(timedban_timeout);
 
-MOD_TEST(timedban)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(timedban)
+MOD_INIT()
 {
-ExtbanInfo extban;
+	ExtbanInfo extban;
 
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 
 	memset(&extban, 0, sizeof(ExtbanInfo));
-	extban.flag = 't';
+	extban.letter = 't';
+	extban.name = "time";
 	extban.options |= EXTBOPT_ACTMODIFIER; /* not really, but ours shouldn't be stacked from group 1 */
-	extban.options |= EXTBOPT_CHSVSMODE; /* so "SVSMODE -nick" will unset affected ~t extbans */
 	extban.options |= EXTBOPT_INVEX; /* also permit timed invite-only exceptions (+I) */
 	extban.conv_param = timedban_extban_conv_param;
 	extban.is_ok = timedban_extban_is_ok;
 	extban.is_banned = timedban_is_banned;
+	extban.is_banned_events = BANCHK_ALL;
 
 	if (!ExtbanAdd(modinfo->handle, extban))
 	{
@@ -91,41 +90,42 @@ ExtbanInfo extban;
 		return MOD_FAILED;
 	}
                 
-	EventAddEx(modinfo->handle, "timedban_timeout", TIMEDBAN_TIMER, 0, timedban_timeout, NULL);
+	EventAdd(modinfo->handle, "timedban_timeout", timedban_timeout, NULL, TIMEDBAN_TIMER*1000, 0);
 
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(timedban)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(timedban)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
 /** Generic helper for our conv_param extban function.
  * Mostly copied from clean_ban_mask()
+ * FIXME: Figure out why we have this one at all and not use conv_param? ;)
  */
-char *generic_clean_ban_mask(char *mask)
+const char *generic_clean_ban_mask(BanContext *b, Extban *extban)
 {
 	char *cp, *x;
 	char *user;
 	char *host;
-	Extban *p;
 	static char maskbuf[512];
+	char *mask;
 
 	/* Work on a copy */
-	strlcpy(maskbuf, mask, sizeof(maskbuf));
+	strlcpy(maskbuf, b->banstr, sizeof(maskbuf));
 	mask = maskbuf;
 
-	cp = index(mask, ' ');
+	cp = strchr(mask, ' ');
 	if (cp)
 		*cp = '\0';
 
-	/* Strip any ':' at beginning since that would cause a desynch */
+	/* Strip any ':' at beginning since that would cause a desync */
 	for (; (*mask && (*mask == ':')); mask++);
 	if (!*mask)
 		return NULL;
@@ -136,13 +136,24 @@ char *generic_clean_ban_mask(char *mask)
 			return NULL;
 
 	/* Extended ban? */
-	if ((*mask == '~') && mask[1] && (mask[2] == ':'))
+	if (is_extended_ban(mask))
 	{
-		p = findmod_by_bantype(mask[1]);
-		if (!p)
+		const char *nextbanstr;
+		Extban *extban = findmod_by_bantype(mask, &nextbanstr);
+		if (!extban)
 			return NULL; /* reject unknown extban */
-		if (p->conv_param)
-			return p->conv_param(mask);
+		if (extban->conv_param)
+		{
+			const char *ret;
+			static char retbuf[512];
+			BanContext *newb = safe_alloc(sizeof(BanContext));
+			newb->banstr = nextbanstr;
+			newb->conv_options = b->conv_options;
+			ret = extban->conv_param(newb, extban);
+			ret = prefix_with_extban(ret, newb, extban, retbuf, sizeof(retbuf));
+			safe_free(newb);
+			return ret;
+		}
 		/* else, do some basic sanity checks and cut it off at 80 bytes */
 		if ((mask[1] != ':') || (mask[2] == '\0'))
 		    return NULL; /* require a ":<char>" after extban type */
@@ -154,9 +165,9 @@ char *generic_clean_ban_mask(char *mask)
 	if ((*mask == '~') && !strchr(mask, '@'))
 		return NULL; /* not an extended ban and not a ~user@host ban either. */
 
-	if ((user = index((cp = mask), '!')))
+	if ((user = strchr((cp = mask), '!')))
 		*user++ = '\0';
-	if ((host = rindex(user ? user : cp, '@')))
+	if ((host = strrchr(user ? user : cp, '@')))
 	{
 		*host++ = '\0';
 
@@ -164,14 +175,14 @@ char *generic_clean_ban_mask(char *mask)
 			return make_nick_user_host(NULL, trim_str(cp,USERLEN), 
 				trim_str(host,HOSTLEN));
 	}
-	else if (!user && index(cp, '.'))
+	else if (!user && strchr(cp, '.'))
 		return make_nick_user_host(NULL, NULL, trim_str(cp,HOSTLEN));
 	return make_nick_user_host(trim_str(cp,NICKLEN), trim_str(user,USERLEN), 
 		trim_str(host,HOSTLEN));
 }
 
 /** Convert ban to an acceptable format (or return NULL to fully reject it) */
-char *timedban_extban_conv_param(char *para_in)
+const char *timedban_extban_conv_param(BanContext *b, Extban *extban)
 {
 	static char retbuf[MAX_LENGTH+1];
 	char para[MAX_LENGTH+1];
@@ -179,13 +190,13 @@ char *timedban_extban_conv_param(char *para_in)
 	char *durationstr; /**< Duration, such as '5' */
 	int duration;
 	char *matchby; /**< Matching method, such as 'n!u@h' */
-	char *newmask; /**< Cleaned matching method, such as 'n!u@h' */
+	const char *newmask; /**< Cleaned matching method, such as 'n!u@h' */
 	static int timedban_extban_conv_param_recursion = 0;
 	
 	if (timedban_extban_conv_param_recursion)
 		return NULL; /* reject: recursion detected! */
 
-	strlcpy(para, para_in+3, sizeof(para)); /* work on a copy (and truncate it) */
+	strlcpy(para, b->banstr, sizeof(para)); /* work on a copy (and truncate it) */
 	
 	/* ~t:duration:n!u@h   for direct matching
 	 * ~t:duration:~x:.... when calling another bantype
@@ -205,82 +216,78 @@ char *timedban_extban_conv_param(char *para_in)
 	strlcpy(tmpmask, matchby, sizeof(tmpmask));
 	timedban_extban_conv_param_recursion++;
 	//newmask = extban_conv_param_nuh_or_extban(tmpmask);
-	newmask = generic_clean_ban_mask(tmpmask);
+	b->banstr = matchby; // this was previously 'tmpmask' but then it's a copy-copy-copy.. :D
+	newmask = generic_clean_ban_mask(b, extban);
 	timedban_extban_conv_param_recursion--;
 	if (!newmask || (strlen(newmask) <= 1))
 		return NULL;
 
-	snprintf(retbuf, sizeof(retbuf), "~t:%d:%s", duration, newmask);
+	//snprintf(retbuf, sizeof(retbuf), "~t:%d:%s", duration, newmask);
+	snprintf(retbuf, sizeof(retbuf), "%d:%s", duration, newmask);
 	return retbuf;
 }
 
-int timedban_extban_syntax(aClient *sptr, int checkt, char *reason)
+int timedban_extban_syntax(Client *client, int checkt, char *reason)
 {
-	if (MyClient(sptr) && (checkt == EXBCHK_PARAM))
+	if (MyUser(client) && (checkt == EXBCHK_PARAM))
 	{
-		sendnotice(sptr, "Error when setting timed ban: %s", reason);
-		sendnotice(sptr, " Syntax: +b ~t:duration:mask");
-		sendnotice(sptr, "Example: +b ~t:5:nick!user@host");
-		sendnotice(sptr, "Duration is the time in minutes after which the ban is removed (1-9999)");
-		sendnotice(sptr, "Valid masks are: nick!user@host or another extban type such as ~a, ~c, ~S, ..");
+		sendnotice(client, "Error when setting timed ban: %s", reason);
+		sendnotice(client, " Syntax: +b ~t:duration:mask");
+		sendnotice(client, "Example: +b ~t:5:nick!user@host");
+		sendnotice(client, "Duration is the time in minutes after which the ban is removed (1-9999)");
+		sendnotice(client, "Valid masks are: nick!user@host or another extban type such as ~a, ~c, ~S, ..");
 	}
 	return 0; /* FAIL: ban rejected */
 }
 
 /** Generic helper for sub-bans, used by our "is this ban ok?" function */
-int generic_ban_is_ok(aClient *sptr, aChannel *chptr, char *mask, int checkt, int what, int what2)
+int generic_ban_is_ok(BanContext *b)
 {
-	if ((mask[0] == '~') && MyClient(sptr))
+	if ((b->banstr[0] == '~') && MyUser(b->client))
 	{
-		Extban *p;
+		Extban *extban;
+		const char *nextbanstr;
 
 		/* This portion is copied from clean_ban_mask() */
-		if (mask[1] && (mask[2] == ':') &&
-		    RESTRICT_EXTENDEDBANS && MyClient(sptr) &&
-		    !ValidatePermissionsForPath("immune:restrict-extendedbans",sptr,NULL,NULL,NULL))
+		if (is_extended_ban(b->banstr) && MyUser(b->client))
 		{
-			if (!strcmp(RESTRICT_EXTENDEDBANS, "*"))
+			if (RESTRICT_EXTENDEDBANS && !ValidatePermissionsForPath("immune:restrict-extendedbans",b->client,NULL,NULL,NULL))
 			{
-				if (checkt == EXBCHK_ACCESS_ERR)
-					sendnotice(sptr, "Setting/removing of extended bans has been disabled");
-				return 0; /* REJECT */
+				if (!strcmp(RESTRICT_EXTENDEDBANS, "*"))
+				{
+					if (b->is_ok_check == EXBCHK_ACCESS_ERR)
+						sendnotice(b->client, "Setting/removing of extended bans has been disabled");
+					return 0; /* REJECT */
+				}
+				if (strchr(RESTRICT_EXTENDEDBANS, b->banstr[1]))
+				{
+					if (b->is_ok_check == EXBCHK_ACCESS_ERR)
+						sendnotice(b->client, "Setting/removing of extended bantypes '%s' has been disabled", RESTRICT_EXTENDEDBANS);
+					return 0; /* REJECT */
+				}
 			}
-			if (strchr(RESTRICT_EXTENDEDBANS, mask[1]))
+			/* And next is inspired by cmd_mode */
+			extban = findmod_by_bantype(b->banstr, &nextbanstr);
+			if (extban && extban->is_ok)
 			{
-				if (checkt == EXBCHK_ACCESS_ERR)
-					sendnotice(sptr, "Setting/removing of extended bantypes '%s' has been disabled", RESTRICT_EXTENDEDBANS);
-				return 0; /* REJECT */
+				b->banstr = nextbanstr;
+				if ((b->is_ok_check == EXBCHK_ACCESS) || (b->is_ok_check == EXBCHK_ACCESS_ERR))
+				{
+					if (!extban->is_ok(b) &&
+					    !ValidatePermissionsForPath("channel:override:mode:extban",b->client,NULL,b->channel,NULL))
+					{
+						return 0; /* REJECT */
+					}
+				} else
+				if (b->is_ok_check == EXBCHK_PARAM)
+				{
+					if (!extban->is_ok(b))
+					{
+						return 0; /* REJECT */
+					}
+				}
 			}
 		}
-		/* End of portion */
-
-		/* This portion is inspired by m_mode */
-		p = findmod_by_bantype(mask[1]);
-		if (checkt == EXBCHK_ACCESS)
-		{
-			if (p && p->is_ok && !p->is_ok(sptr, chptr, mask, EXBCHK_ACCESS, what, what2) &&
-			    !ValidatePermissionsForPath("channel:override:mode:extban",sptr,NULL,chptr,NULL))
-			{
-				return 0; /* REJECT */
-			}
-		} else
-		if (checkt == EXBCHK_ACCESS_ERR)
-		{
-			if (p && p->is_ok && !p->is_ok(sptr, chptr, mask, EXBCHK_ACCESS, what, what2) &&
-			    !ValidatePermissionsForPath("channel:override:mode:extban",sptr,NULL,chptr,NULL))
-			{
-				p->is_ok(sptr, chptr, mask, EXBCHK_ACCESS_ERR, what, what2);
-				return 0; /* REJECT */
-			}
-		} else
-		if (checkt == EXBCHK_PARAM)
-		{
-			if (p && p->is_ok && !p->is_ok(sptr, chptr, mask, EXBCHK_PARAM, what, what2))
-			{
-				return 0; /* REJECT */
-			}
-		}
-		/* End of portion */
 	}
 	
 	/* ACCEPT:
@@ -292,7 +299,7 @@ int generic_ban_is_ok(aClient *sptr, aChannel *chptr, char *mask, int checkt, in
 }
 
 /** Validate ban ("is this ban ok?") */
-int timedban_extban_is_ok(aClient* sptr, aChannel* chptr, char* para_in, int checkt, int what, int what2)
+int timedban_extban_is_ok(BanContext *b)
 {
 	char para[MAX_LENGTH+1];
 	char tmpmask[MAX_LENGTH+1];
@@ -304,13 +311,13 @@ int timedban_extban_is_ok(aClient* sptr, aChannel* chptr, char* para_in, int che
 	int res;
 
 	/* Always permit deletion */
-	if (what == MODE_DEL)
+	if (b->what == MODE_DEL)
 		return 1;
 
 	if (timedban_extban_is_ok_recursion)
 		return 0; /* Recursion detected (~t:1:~t:....) */
 
-	strlcpy(para, para_in+3, sizeof(para)); /* work on a copy (and truncate it) */
+	strlcpy(para, b->banstr, sizeof(para)); /* work on a copy (and truncate it) */
 	
 	/* ~t:duration:n!u@h   for direct matching
 	 * ~t:duration:~x:.... when calling another bantype
@@ -319,18 +326,19 @@ int timedban_extban_is_ok(aClient* sptr, aChannel* chptr, char* para_in, int che
 	durationstr = para;
 	matchby = strchr(para, ':');
 	if (!matchby || !matchby[1])
-		return timedban_extban_syntax(sptr, checkt, "Invalid syntax");
+		return timedban_extban_syntax(b->client, b->is_ok_check, "Invalid syntax");
 	*matchby++ = '\0';
 
 	duration = atoi(durationstr);
 
 	if ((duration <= 0) || (duration > TIMEDBAN_MAX_TIME))
-		return timedban_extban_syntax(sptr, checkt, "Invalid duration time");
+		return timedban_extban_syntax(b->client, b->is_ok_check, "Invalid duration time");
 
 	strlcpy(tmpmask, matchby, sizeof(tmpmask));
 	timedban_extban_is_ok_recursion++;
-	//res = extban_is_ok_nuh_extban(sptr, chptr, tmpmask, checkt, what, what2);
-	res = generic_ban_is_ok(sptr, chptr, tmpmask, checkt, what, what2);
+	//res = extban_is_ok_nuh_extban(b->client, b->channel, tmpmask, b->is_ok_check, b->what, b->ban_type);
+	b->banstr = tmpmask;
+	res = generic_ban_is_ok(b);
 	timedban_extban_is_ok_recursion--;
 	if (res == 0)
 	{
@@ -338,41 +346,48 @@ int timedban_extban_is_ok(aClient* sptr, aChannel* chptr, char* para_in, int che
 		 * invalid n!u@h syntax, unknown (sub)extbantype,
 		 * disabled extban type in conf, too much recursion, etc.
 		 */
-		return timedban_extban_syntax(sptr, checkt, "Invalid matcher");
+		return timedban_extban_syntax(b->client, b->is_ok_check, "Invalid matcher");
 	}
 
 	return 1; /* OK */
 }
 
 /** Check if the user is currently banned */
-int timedban_is_banned(aClient *sptr, aChannel *chptr, char *ban, int chktype)
+int timedban_is_banned(BanContext *b)
 {
-	if (strncmp(ban, "~t:", 3))
-		return 0; /* not for us */
-	ban = strchr(ban+3, ':'); /* skip time argument */
-	if (!ban)
+	b->banstr = strchr(b->banstr, ':'); /* skip time argument */
+	if (!b->banstr)
 		return 0; /* invalid fmt */
-	ban++;
+	b->banstr++; /* skip over final semicolon */
 
-	return ban_check_mask(sptr, chptr, ban, chktype, 0);
+	return ban_check_mask(b);
 }
 
-/** Helper to check if the ban has been expired */
+/** Helper to check if the ban has been expired.
+ */
 int timedban_has_ban_expired(Ban *ban)
 {
-char *banstr = ban->banstr;
-char *p;
-int t;
-TS expire_on;
+	char *banstr = ban->banstr;
+	char *p1, *p2;
+	int t;
+	time_t expire_on;
 
-	if (strncmp(banstr, "~t:", 3))
+	/* The caller has only performed a very light check (string starting
+	 * with ~t, in the interest of performance), so we don't know yet if
+	 * it REALLY is a timed ban. We check that first here...
+	 */
+	if (!strncmp(banstr, "~t:", 3))
+		p1 = banstr + 3;
+	else if (!strncmp(banstr, "~time:", 6))
+		p1 = banstr + 6;
+	else
 		return 0; /* not for us */
-	p = strchr(banstr+3, ':'); /* skip time argument */
-	if (!p)
+	p2 = strchr(p1+1, ':'); /* skip time argument */
+	if (!p2)
 		return 0; /* invalid fmt */
-	*p = '\0'; /* danger.. must restore!! */
-	t = atoi(banstr+3);
-	*p = ':'; /* restored.. */
+	*p2 = '\0'; /* danger.. must restore!! */
+	t = atoi(p1);
+	*p2 = ':'; /* restored.. */
 	
 	expire_on = ban->when + (t * 60) - TIMEDBAN_TIMER_DELTA;
 	
@@ -387,56 +402,59 @@ static char pbuf[512];
 /** This removes any expired timedbans */
 EVENT(timedban_timeout)
 {
-	aChannel *chptr;
+	Channel *channel;
 	Ban *ban, *nextban;
 	static int current_iteration = 0;
 
 	if (++current_iteration >= TIMEDBAN_TIMER_ITERATION_SPLIT)
 		current_iteration = 0;
 
-	for (chptr = channel; chptr; chptr = chptr->nextch)
+	for (channel = channels; channel; channel = channel->nextch)
 	{
 		/* This is a very quick check, at the cost of it being
 		 * biased since there's always a tendency of more channel
 		 * names to start with one specific letter. But hashing
 		 * is too costly. So we stick with this. It should be
-		 * good enough. Alternative would be some chptr->id value.
+		 * good enough. Alternative would be some channel->id value.
 		 */
-		if (((unsigned int)chptr->chname[1] % TIMEDBAN_TIMER_ITERATION_SPLIT) != current_iteration)
+		if (((unsigned int)channel->name[1] % TIMEDBAN_TIMER_ITERATION_SPLIT) != current_iteration)
 			continue; /* not this time, maybe next */
 
 		*mbuf = *pbuf = '\0';
-		for (ban = chptr->banlist; ban; ban=nextban)
+		for (ban = channel->banlist; ban; ban=nextban)
 		{
 			nextban = ban->next;
-			if (!strncmp(ban->banstr, "~t:", 3) && timedban_has_ban_expired(ban))
+			if (!strncmp(ban->banstr, "~t", 2) && timedban_has_ban_expired(ban))
 			{
-				add_send_mode_param(chptr, &me, '-',  'b', ban->banstr);
-				del_listmode(&chptr->banlist, chptr, ban->banstr);
+				add_send_mode_param(channel, &me, '-',  'b', ban->banstr);
+				del_listmode(&channel->banlist, channel, ban->banstr);
 			}
 		}
-		for (ban = chptr->exlist; ban; ban=nextban)
+		for (ban = channel->exlist; ban; ban=nextban)
 		{
 			nextban = ban->next;
-			if (!strncmp(ban->banstr, "~t:", 3) && timedban_has_ban_expired(ban))
+			if (!strncmp(ban->banstr, "~t", 2) && timedban_has_ban_expired(ban))
 			{
-				add_send_mode_param(chptr, &me, '-',  'e', ban->banstr);
-				del_listmode(&chptr->exlist, chptr, ban->banstr);
+				add_send_mode_param(channel, &me, '-',  'e', ban->banstr);
+				del_listmode(&channel->exlist, channel, ban->banstr);
 			}
 		}
-		for (ban = chptr->invexlist; ban; ban=nextban)
+		for (ban = channel->invexlist; ban; ban=nextban)
 		{
 			nextban = ban->next;
-			if (!strncmp(ban->banstr, "~t:", 3) && timedban_has_ban_expired(ban))
+			if (!strncmp(ban->banstr, "~t", 2) && timedban_has_ban_expired(ban))
 			{
-				add_send_mode_param(chptr, &me, '-',  'I', ban->banstr);
-				del_listmode(&chptr->invexlist, chptr, ban->banstr);
+				add_send_mode_param(channel, &me, '-',  'I', ban->banstr);
+				del_listmode(&channel->invexlist, channel, ban->banstr);
 			}
 		}
 		if (*pbuf)
 		{
-			sendto_channel_butserv(chptr, &me, ":%s MODE %s %s %s", me.name, chptr->chname, mbuf, pbuf);
-			sendto_server(NULL, 0, 0, ":%s MODE %s %s %s 0", me.name, chptr->chname, mbuf, pbuf);
+			MessageTag *mtags = NULL;
+			new_message(&me, NULL, &mtags);
+			sendto_channel(channel, &me, NULL, 0, 0, SEND_LOCAL, mtags, ":%s MODE %s %s %s", me.name, channel->name, mbuf, pbuf);
+			sendto_server(NULL, 0, 0, mtags, ":%s MODE %s %s %s 0", me.id, channel->name, mbuf, pbuf);
+			free_message_tags(mtags);
 			*pbuf = 0;
 		}
 	}
@@ -446,7 +464,7 @@ EVENT(timedban_timeout)
  #error "add_send_mode_param() is not made for MODEBUFLEN > 512"
 #endif
 
-void add_send_mode_param(aChannel *chptr, aClient *from, char what, char mode, char *param) {
+void add_send_mode_param(Channel *channel, Client *from, char what, char mode, char *param) {
 	static char *modes = NULL, lastwhat;
 	static short count = 0;
 	short send = 0;
@@ -480,21 +498,27 @@ void add_send_mode_param(aChannel *chptr, aClient *from, char what, char mode, c
 	if (count == MAXMODEPARAMS)
 		send = 1;
 
-	if (send) {
-		sendto_channel_butserv(chptr, &me, ":%s MODE %s %s %s", me.name, chptr->chname, mbuf, pbuf);
-		sendto_server(NULL, 0, 0, ":%s MODE %s %s %s 0", me.name, chptr->chname, mbuf, pbuf);
+	if (send)
+	{
+		MessageTag *mtags = NULL;
+
+		new_message(&me, NULL, &mtags);
+		sendto_channel(channel, &me, NULL, 0, 0, SEND_LOCAL, mtags, ":%s MODE %s %s %s", me.name, channel->name, mbuf, pbuf);
+		sendto_server(NULL, 0, 0, mtags, ":%s MODE %s %s %s 0", me.id, channel->name, mbuf, pbuf);
+		free_message_tags(mtags);
 		send = 0;
 		*pbuf = 0;
 		modes = mbuf;
 		*modes++ = what;
 		lastwhat = what;
-		if (count != MAXMODEPARAMS) {
+		if (count != MAXMODEPARAMS)
+		{
 			strlcpy(pbuf, param, sizeof(pbuf));
 			*modes++ = mode;
 			count = 1;
-		}
-		else 
+		} else {
 			count = 0;
+		}
 		*modes = 0;
 	}
 }
