@@ -85,9 +85,6 @@ UINT WM_TASKBARCREATED, WM_FINDMSGSTRING;
 FARPROC lpfnOldWndProc;
 HMENU hContext;
 char OSName[OSVER_SIZE];
-#ifdef USE_LIBCURL
-extern char *find_loaded_remote_include(char *url);
-#endif 
 
 void TaskBarCreated() 
 {
@@ -169,8 +166,9 @@ LRESULT RESubClassFunc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
 
 int DoCloseUnreal(HWND hWnd)
 {
-	ircd_log(LOG_ERROR, "Stopping UnrealIRCd...");
-	loop.ircd_terminating = 1;
+	unreal_log(ULOG_INFO, "main", "UNREALIRCD_STOP", NULL,
+	           "Terminating server (process termination requested or GUI window closed)");
+	loop.terminating = 1;
 	unload_all_modules();
 	DestroyWindow(hWnd);
 	TerminateProcess(GetCurrentProcess(), 0);
@@ -190,7 +188,6 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 	MSG msg;
 	unsigned char *s;
 	HWND hWnd;
-	WSADATA WSAData;
 	HICON hIcon;
 	SC_HANDLE hService, hSCManager;
 	SERVICE_TABLE_ENTRY DispatchTable[] = 
@@ -201,7 +198,7 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 	DWORD need;
 	
 	/* Go one level up, since we are currently in the bin\ subdir
-	 * and we want to be in (f.e.) "C:\Program Files\UnrealIRCd 5"
+	 * and we want to be in (f.e.) "C:\Program Files\UnrealIRCd 6"
 	 */
 	chdir("..");
 
@@ -249,15 +246,10 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 	WM_TASKBARCREATED = RegisterWindowMessage("TaskbarCreated");
 	WM_FINDMSGSTRING = RegisterWindowMessage(FINDMSGSTRING);
 	atexit(CleanUp);
-	if(!LoadLibrary("riched20.dll"))
+	if (!LoadLibrary("riched20.dll"))
 		LoadLibrary("riched32.dll");
 	InitDebug();
-
-	if (WSAStartup(MAKEWORD(1, 1), &WSAData) != 0)
-	{
-		MessageBox(NULL, "Unable to initialize WinSock", "UnrealIRCD Initalization Error", MB_OK);
-		return FALSE;
-	}
+	init_winsock();
 	hInst = hInstance; 
 
 	MainDlgBackground = CreateSolidBrush(RGB(75, 134, 238)); /* Background of main dialog */
@@ -356,6 +348,7 @@ LRESULT CALLBACK MainDLG(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 					DestroyMenu(hLogs);
 					hLogs = CreatePopupMenu();
 					AppendMenu(hConfig, MF_STRING, IDM_CONF, CPATH);
+#if 0
 					if (conf_log) 
 					{
 						ConfigItem_log *logs;
@@ -366,22 +359,7 @@ LRESULT CALLBACK MainDLG(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 						}
 					}
 					AppendMenu(hConfig, MF_SEPARATOR, 0, NULL);
-					if (conf_include) 
-					{
-						ConfigItem_include *inc;
-						for (inc = conf_include; inc; inc = inc->next)
-						{
-							if (inc->flag.type & INCLUDE_NOTLOADED)
-								continue;
-#ifdef USE_LIBCURL
-							if (inc->flag.type & INCLUDE_REMOTE)
-								AppendMenu(hConfig, MF_STRING, i++, inc->url);
-							else
 #endif
-							AppendMenu(hConfig, MF_STRING, i++, inc->file);
-						}
-						AppendMenu(hConfig, MF_SEPARATOR, 0, NULL);
-					}
 					if (conf_files)
 					{
 						AppendMenu(hConfig, MF_STRING, IDM_MOTD, conf_files->motd_file);
@@ -464,6 +442,7 @@ LRESULT CALLBACK MainDLG(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 				hLogs = CreatePopupMenu();
 
 				AppendMenu(hConfig, MF_STRING, IDM_CONF, CPATH);
+#if 0
 				if (conf_log) 
 				{
 					ConfigItem_log *logs;
@@ -474,22 +453,7 @@ LRESULT CALLBACK MainDLG(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 					}
 				}
 				AppendMenu(hConfig, MF_SEPARATOR, 0, NULL);
-
-				if (conf_include) 
-				{
-					ConfigItem_include *inc;
-					for (inc = conf_include; inc; inc = inc->next)
-					{
-#ifdef USE_LIBCURL
-						if (inc->flag.type & INCLUDE_REMOTE)
-							AppendMenu(hConfig, MF_STRING, i++, inc->url);
-						else
 #endif
-						AppendMenu(hConfig, MF_STRING, i++, inc->file);
-					}
-					AppendMenu(hConfig, MF_SEPARATOR, 0, NULL);
-				}
-
 				if (conf_files)
 				{
 					AppendMenu(hConfig, MF_STRING, IDM_MOTD, conf_files->motd_file);
@@ -548,20 +512,13 @@ LRESULT CALLBACK MainDLG(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 				else 
 				{
 					GetMenuString(hConfig,LOWORD(wParam), path, MAX_PATH, MF_BYCOMMAND);
-#ifdef USE_LIBCURL
-					if (url_is_valid(path))
-					{
-						char *file = find_loaded_remote_include(path);
-						DialogBoxParam(hInst, "FromVar", hDlg, (DLGPROC)FromFileReadDLG, (LPARAM)file);
-					}
-					else
-#endif
+					if (!url_is_valid(path))
 						DialogBoxParam(hInst, "FromFile", hDlg, (DLGPROC)FromFileDLG, (LPARAM)path);
 				}
 				return FALSE;
 			}
 
-			if (!loop.ircd_booted)
+			if (!loop.booted)
 			{
 				MessageBox(NULL, "UnrealIRCd not booted due to configuration errors. "
 				                 "Check other window for error details. Then close that window, "
@@ -579,31 +536,25 @@ LRESULT CALLBACK MainDLG(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 					return AskCloseUnreal(hDlg);
 				case IDM_RHALL:
 					MessageBox(NULL, "Rehashing all files", "Rehashing", MB_OK);
-					sendto_realops("Rehashing all files via the console");
-					rehash(&me,0);
-					reread_motdsandrules();
+					request_rehash(NULL);
 					break;
 				case IDM_RHCONF:
 					MessageBox(NULL, "Rehashing the Config file", "Rehashing", MB_OK);
-					sendto_realops("Rehashing the Config file via the console");
-					rehash(&me,0);
+					request_rehash(NULL);
 					break;
 				case IDM_RHMOTD: 
 				{
 					MessageBox(NULL, "Rehashing all MOTD and Rules files", "Rehashing", MB_OK);
 					rehash_motdrules();
-					sendto_realops("Rehashing all MOTD and Rules files via the console");
 					break;
 				}
 				case IDM_RHOMOTD:
 					MessageBox(NULL, "Rehashing the OperMOTD", "Rehashing", MB_OK);
 					read_motd(conf_files->opermotd_file, &opermotd);
-					sendto_realops("Rehashing the OperMOTD via the console");
 					break;
 				case IDM_RHBMOTD:
 					MessageBox(NULL, "Rehashing the BotMOTD", "Rehashing", MB_OK);
 					read_motd(conf_files->botmotd_file, &botmotd);
-					sendto_realops("Rehashing the BotMOTD via the console");
 					break;
 				case IDM_LICENSE: 
 					DialogBox(hInst, "FromVar", hDlg, (DLGPROC)LicenseDLG);
@@ -1031,7 +982,7 @@ void win_map(Client *server, HWND hwTreeView, short remap)
 	for (lp = Servers; lp; lp = lp->next)
         {
                 acptr = lp->value.client;
-                if (acptr->srvptr != server)
+                if (acptr->uplink != server)
                         continue;
                 win_map(acptr, hwTreeView, 0);
         }
@@ -1091,27 +1042,6 @@ void win_error()
 {
 	if (errors && !IsService)
 		DialogBox(hInst, "ConfigError", hwIRCDWnd, (DLGPROC)ConfigErrorDLG);
-	if (need_34_upgrade)
-	{
-		need_34_upgrade = 0; /* anti-recursion. yes, is needed. */
-		if (MessageBox(NULL, 
-		               "Shall I try to upgrade your configuration files to UnrealIRCd 4 format?",
-		               "3.2.x configuration detected",
-		               MB_YESNO|MB_ICONQUESTION) == IDNO)
-		{
-			 return;
-		}
-		else 
-		{
-			update_conf();
-			MessageBox(NULL,
-			           "Configuration file(s) upgraded! In next screen you can see what I did (just for reference). "
-			           "After that, simply try to start UnrealIRCd again and see if it loads.",
-			           "Configuration upgrade",
-			           MB_OK);
-			DialogBox(hInst, "ConfigError", hwIRCDWnd, (DLGPROC)ConfigErrorDLG);
-		}
-	}
 }
 
 LRESULT CALLBACK ConfigErrorDLG(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) 
